#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.gallery.interfaces import IGalleryFile
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_file.interfaces import DELETED_FILE, IResponsiveImage
from pyams_form.interfaces.form import IFormContextPermissionChecker
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent

# import packages
from persistent import Persistent
from pyams_file.property import FileProperty
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.traversing import get_parent
from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.lifecycleevent import ObjectModifiedEvent
from zope.container.contained import Contained
from zope.interface import implementer, alsoProvides
from zope.schema.fieldproperty import FieldProperty


#
# Gallery file
#

@implementer(IGalleryFile)
class GalleryFile(Persistent, Contained):
    """Gallery file info"""

    title = FieldProperty(IGalleryFile['title'])
    alt_title = FieldProperty(IGalleryFile['alt_title'])
    _data = FileProperty(IGalleryFile['data'])
    description = FieldProperty(IGalleryFile['description'])
    author = FieldProperty(IGalleryFile['author'])
    author_comments = FieldProperty(IGalleryFile['author_comments'])
    pif_number = FieldProperty(IGalleryFile['pif_number'])
    sound = FileProperty(IGalleryFile['sound'])
    sound_title = FieldProperty(IGalleryFile['sound_title'])
    sound_description = FieldProperty(IGalleryFile['sound_description'])
    visible = FieldProperty(IGalleryFile['visible'])

    @property
    def data(self):
        return self._data

    @data.setter
    def data(self, value):
        self._data = value
        if (value is not None) and (value is not DELETED_FILE):
            alsoProvides(self._data, IResponsiveImage)


@adapter_config(context=IGalleryFile, provides=IFormContextPermissionChecker)
class GalleryFilePermissionChecker(ContextAdapter):
    """Gallery file permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IWfSharedContent)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IGalleryFile)
def handle_added_gallery_file(event):
    """Handle added gallery file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IGalleryFile)
def handle_modified_gallery_file(event):
    """Handle modified gallery file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IGalleryFile)
def handle_removed_gallery_file(event):
    """Handle removed gallery file"""
    content = get_parent(event.object, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))
