#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IBaseParagraph
from zope.container.interfaces import IOrderedContainer
from zope.contentprovider.interfaces import IContentProvider

# import packages
from pyams_file.schema import FileField, ImageField
from pyams_i18n.schema import I18nTextLineField, I18nTextField
from zope.annotation.interfaces import IAttributeAnnotatable
from zope.container.constraints import contains, containers
from zope.interface import Interface, Attribute
from zope.schema import Bool, TextLine, Choice

from pyams_content import _


GALLERY_CONTAINER_KEY = 'pyams_content.gallery'


class IGalleryFile(Interface):
    """Gallery file marker interface"""

    containers('.IGallery')

    title = I18nTextLineField(title=_("Legend"),
                              required=False)

    alt_title = I18nTextLineField(title=_("Accessibility title"),
                                  description=_("Alternate title used to describe image content"),
                                  required=False)

    data = ImageField(title=_("Image data"),
                      description=_("Image content"),
                      required=True)

    description = I18nTextField(title=_("Description"),
                                required=False)

    author = TextLine(title=_("Author"),
                      description=_("Name of document's author"),
                      required=False)

    author_comments = I18nTextField(title=_("Author's comments"),
                                    description=_("Comments relatives to author's rights management"),
                                    required=False)

    pif_number = TextLine(title=_("PIF number"),
                          description=_("Number used to identify media into national library database"),
                          required=False)

    sound = FileField(title=_("Audio data"),
                      description=_("Sound file associated with the current media"),
                      required=False)

    sound_title = I18nTextLineField(title=_("Sound title"),
                                    description=_("Title of associated sound file"),
                                    required=False)

    sound_description = I18nTextField(title=_("Sound description"),
                                      description=_("Short description of associated sound file"),
                                      required=False)

    visible = Bool(title=_("Visible image?"),
                   description=_("If 'no', this image won't be displayed in front office"),
                   required=True,
                   default=True)


class IBaseGallery(IOrderedContainer, IAttributeAnnotatable):
    """Base gallery interface"""

    title = I18nTextLineField(title=_("Title"),
                              description=_("Gallery title, as shown in front-office"),
                              required=True)

    description = I18nTextField(title=_("Description"),
                                description=_("Gallery description displayed by front-office template"),
                                required=False)

    renderer = Choice(title=_("Gallery style"),
                      vocabulary='PyAMS gallery renderers')

    def append(self, value, notify=True):
        """Append new file to gallery"""

    def get_visible_images(self):
        """Get iterator over visible images"""


class IGallery(IBaseGallery):
    """Gallery interface"""

    contains(IGalleryFile)


class IGalleryRenderer(IContentProvider):
    """Gallery renderer utility interface"""

    label = Attribute("Renderer label")


class IGalleryTarget(IAttributeAnnotatable):
    """Gallery container target marker interface"""


class IGalleryParagraph(IGallery, IBaseParagraph):
    """Gallery paragraph"""
