#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget, IParagraphContainer, \
    IParagraphSummary
from pyams_content.shared.common.interfaces.zmi import IPreviewForm
from pyams_form.interfaces.form import IWidgetsSuffixViewletsManager
from pyams_i18n.interfaces import II18nManager
from pyams_skin.layer import IPyAMSLayer

# import packages
from pyams_template.template import template_config
from pyams_utils.interfaces import VIEW_SYSTEM_PERMISSION
from pyams_viewlet.viewlet import viewlet_config, Viewlet


@viewlet_config(name='paragraphs-preview', context=IParagraphContainerTarget, layer=IPyAMSLayer,
                view=IPreviewForm, manager=IWidgetsSuffixViewletsManager,
                permission=VIEW_SYSTEM_PERMISSION, weight=100)
@template_config(template='templates/preview.pt', layer=IPyAMSLayer)
class ParagraphsContainerPreview(Viewlet):
    """Paragraphs container preview"""

    def __init__(self, context, request, view, manager):
        super(ParagraphsContainerPreview, self).__init__(context, request, view, manager)
        self.paragraphs = [para for para in IParagraphContainer(self.context).values()
                           if para.visible]
        self.languages = II18nManager(self.context).get_languages()

    def render_paragraph(self, paragraph, language=None):
        renderer = self.request.registry.queryMultiAdapter((paragraph, self.request), IParagraphSummary)
        if renderer is not None:
            renderer.language = language
            renderer.update()
            return renderer.render()
