#
# Copyright (c) 2008-2017 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library
from persistent import Persistent

# import interfaces
from hypatia.interfaces import ICatalog
from pyams_content.shared.view.interfaces import IViewInternalReferencesSettings, IWfView, VIEW_REFERENCES_SETTINGS_KEY, \
    IViewQueryFilterExtension, ALWAYS_REFERENCE_MODE
from zope.annotation.interfaces import IAnnotations

# import packages
from hypatia.catalog import CatalogQuery
from hypatia.query import Any
from pyams_catalog.query import CatalogResultSet
from pyams_content.workflow import VISIBLE_STATES
from pyams_utils.adapter import adapter_config, ContextAdapter
from pyams_utils.registry import get_utility
from pyramid.threadlocal import get_current_registry
from zope.container.contained import Contained
from zope.interface import implementer
from zope.lifecycleevent import ObjectCreatedEvent
from zope.location import locate
from zope.schema.fieldproperty import FieldProperty


@implementer(IViewInternalReferencesSettings)
class ViewInternalReferencesSettings(Persistent, Contained):
    """View internal references settings"""

    references = FieldProperty(IViewInternalReferencesSettings['references'])
    references_mode = FieldProperty(IViewInternalReferencesSettings['references_mode'])


@adapter_config(context=IWfView, provides=IViewInternalReferencesSettings)
def ViewInternalReferencesSettingsFactory(view):
    """View internal references settings factory"""
    annotations = IAnnotations(view)
    settings = annotations.get(VIEW_REFERENCES_SETTINGS_KEY)
    if settings is None:
        settings = annotations[VIEW_REFERENCES_SETTINGS_KEY] = ViewInternalReferencesSettings()
        get_current_registry().notify(ObjectCreatedEvent(settings))
        locate(settings, view, '++view:references++')
    return settings


@adapter_config(name='references', context=IWfView, provides=IViewQueryFilterExtension)
class ViewInternalReferencesQueryFilterExtension(ContextAdapter):
    """View internal references filter extension"""

    weight = 999

    def filter(self, context, items):
        settings = IViewInternalReferencesSettings(self.context)
        if not settings.references:
            return items
        if (not items) or (settings.references_mode == ALWAYS_REFERENCE_MODE):
            catalog = get_utility(ICatalog)
            params = Any(catalog['oid'], settings.references) & Any(catalog['workflow_state'], VISIBLE_STATES)
            items.prepend(CatalogResultSet(CatalogQuery(catalog).query(params)))
        return items
