(function($, globals) {

	"use strict";

	var MyAMS = globals.MyAMS;

	var PyAMS_content = {

		/**
		 * Replace given form with new content
		 */
		refreshForm: function(changes) {
			var parent = $('#' + changes.object_name);
			parent.replaceWith($(changes.form));
			parent = $('#' + changes.object_name);
			MyAMS.initContent(parent);
			return parent;
		},

		refreshTable: function(changes) {
			var widget = $('#' + changes.object_name).parent('.ams-widget');
			widget.replaceWith($(changes.table));
			widget = $('#' + changes.object_name).parent('.ams-widget');
			MyAMS.initContent(widget);
			return widget;
		},


		/**
		 * TinyMCE editor configuration
		 */
		TinyMCE: {

			initEditor: function(settings) {
				settings.image_list = PyAMS_content.TinyMCE.getImagesList;
				settings.link_list = PyAMS_content.TinyMCE.getLinksList;
				return settings;
			},

			getImagesList: function(callback) {
				var form = $(document.activeElement).parents('form');
				if (form.exists()) {
					var base = form.attr('data-ams-form-handler') || form.attr('action');
					var target = base.substr(0, base.lastIndexOf('/') + 1);
					return MyAMS.ajax.post(target + 'get-images-list.json', {}, callback);
				}
			},

			getLinksList: function(callback) {
				var form = $(document.activeElement).parents('form');
				if (form.exists()) {
					var base = form.attr('data-ams-form-handler') || form.attr('action');
					var target = base.substr(0, base.lastIndexOf('/')+1);
					return MyAMS.ajax.post(target + 'get-links-list.json', {}, callback);
				}
			}
		},

		/**
		 * User profile management
		 */
		profile: {

			switchFavorite: function() {
				var star = $(this);
				var oid = star.data('sequence-oid');
				MyAMS.ajax.post('switch-user-favorite.json',
								{oid: oid},
								function(result, status) {
									if (result.favorite) {
										star.removeClass('fa-star-o')
											.addClass('fa-star');
									} else {
										star.removeClass('fa-star')
											.addClass('fa-star-o');
									}
								});
			}
		},

		/**
		 * Galleries management
		 */
		galleries: {

			updateImageTitle: function(options) {
				$('img[id="' + options.image_id + '"]').attr('original-title', options.title);
			},

			switchImageVisibility: function(element) {
				return function() {
					var source = $(this);
					var image = source.parents('.image');
					var gallery = image.parents('.gallery');
					MyAMS.ajax.post(gallery.data('ams-location') + '/set-image-visibility.json',
									{object_name: image.data('ams-element-name')},
									function(result, status) {
										if (result.visible) {
											$('i', source).attr('class', 'fa fa-fw fa-eye');
											source.parents('.btn-group').siblings('a.fancyimg').removeClass('not-visible');
										} else {
											$('i', source).attr('class', 'fa fa-fw fa-eye-slash text-danger');
											source.parents('.btn-group').siblings('a.fancyimg').addClass('not-visible');
										}
									});
				};
			},

			setOrder: function(event, ui) {
				if (ui && ui.item.hasClass('already-dropped')) {
					return;
				}
				var gallery = ui.item.parents('.gallery');
				var ids = $('.image', gallery).listattr('data-ams-element-name');
				MyAMS.ajax.post(gallery.data('ams-location') + '/set-images-order.json',
								{images: JSON.stringify(ids)});
			},

			removeFile: function(element) {
				return function() {
					var link = $(this);
					MyAMS.skin.bigBox({
						title: MyAMS.i18n.WARNING,
						content: '<i class="text-danger fa fa-2x fa-bell shake animated"></i>&nbsp; ' + MyAMS.i18n.DELETE_WARNING,
						buttons: MyAMS.i18n.BTN_OK_CANCEL
					}, function(button) {
						if (button === MyAMS.i18n.BTN_OK) {
							var gallery = link.parents('.gallery');
							var location = gallery.data('ams-location');
							var image = link.parents('.image');
							var object_name = image.data('ams-element-name');
							MyAMS.ajax.post(location + '/delete-element.json',
											{object_name: object_name},
											function(result, status) {
												image.remove();
											});
						}
					});
				};
			},

			afterFancyboxLoad: function(current, previous) {
				var element = current.element;
				if (element.hasClass('not-visible')) {
					current.inner.prepend('<div class="hidden-mask"></div>');
				}
			}
		},


		/**
		 * Image map management
		 */
		imgmap: {

			init: function() {
				var image = $(this);
				MyAMS.ajax.check($.fn.canvasAreaDraw,
								 '/--static--/pyams_content/js/jquery-canvasAreaDraw' + MyAMS.devext + '.js',
								 function() {
									image.canvasAreaDraw({imageUrl: image.data('ams-image-url')});
								 });
			},

			initSummary: function() {
				var image = $(this);
				MyAMS.ajax.check($.fn.mapster,
								 '/--static--/pyams_content/js/jquery-imagemapster-1.2.10' + MyAMS.devext + '.js',
								 function() {
									image.mapster({
										fillColor: 'ff0000',
										fillOpacity: 0.35,
										selected: true,
										highlight: true,
										staticState: true
									});
								 });
			}
		},


		/**
		 * Associations management
		 */
		associations: {

			refreshAssociations: function(options) {
				// Reload widget
				var widget = PyAMS_content.refreshTable(options);
				// Check fieldset state
				var legend = widget.siblings('legend');
				if (legend.parents('fieldset:first').hasClass('switched')) {
					legend.click();
				}
			},

			switchVisibility: function (element) {
				return function () {
					var source = $(this);
					var association = source.parents('tr');
					var container = association.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/set-association-visibility.json',
						{object_name: association.data('ams-element-name')},
						function (result, status) {
							if (result.visible) {
								$('i', source).attr('class', 'fa fa-fw fa-eye');
							} else {
								$('i', source).attr('class', 'fa fa-fw fa-eye-slash text-danger');
							}
						});
				};
			}
		},


		/**
		 * Paragraphs management
		 */
		paragraphs: {

			/* Store and restore paragraphs state on reload */
			preReload: function() {
				PyAMS_content.paragraphs.switched = $('i.switch.fa-minus-square-o', '#paragraphs_list').parents('tr').listattr('id');
			},

			postReload: function() {
				$(PyAMS_content.paragraphs.switched).each(function() {
					$('i.switch.fa-plus-square-o', '[id="' + this + '"]').parents('div:first').click();
				});
				delete PyAMS_content.paragraphs.switched;
			},

			switchVisibility: function(element) {
				return function() {
					var source = $(this);
					var para = source.parents('tr');
					var container = para.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/set-paragraph-visibility.json',
									{object_name: para.data('ams-element-name')},
									function(result, status) {
										if (result.visible) {
											$('i', source).attr('class', 'fa fa-fw fa-eye');
										} else {
											$('i', source).attr('class', 'fa fa-fw fa-eye-slash text-danger');
										}
									});
				};
			},

			refreshParagraph: function(changes) {
				var container = $('table[id="paragraphs_list"]');
				var para = $('tr[data-ams-element-name="' + changes.object_name + '"]', container);
				$('span.title', para).html(changes.title || ' - - - - - - - -');
			},

			switchEditor: function(element) {
				var source = $(this);
				var switcher = $('i.switch', source);
				var td = source.parents('td');
				var editor = $('.editor', td);
				var para = source.parents('tr');
				if (switcher.hasClass('fa-plus-square-o')) {
					var container = para.parents('table');
					editor.html('<h1 class="loading"><i class="fa fa-2x fa-gear fa-spin"></i></h1>');
					MyAMS.ajax.post(container.data('ams-location') + '/get-paragraph-editor.json',
									{object_name: para.data('ams-element-name')},
									function(result) {
										editor.html(result);
										if (result) {
											MyAMS.initContent(editor);
											switcher.removeClass('fa-plus-square-o')
													.addClass('fa-minus-square-o');
											para.data('ams-disabled-handlers', true);
										}
									});
				} else {
					MyAMS.skin.cleanContainer(editor);
					editor.empty();
					switcher.removeClass('fa-minus-square-o')
							.addClass('fa-plus-square-o');
					para.removeData('ams-disabled-handlers');
				}
			},

			switchAllEditors: function(element) {
				var source = $(this);
				var switcher = $('i', source);
				var container = source.parents('table');
				if (switcher.hasClass('fa-plus-square-o')) {
					switcher.removeClass('fa-plus-square-o')
							.addClass('fa-cog fa-spin');
					MyAMS.ajax.post(container.data('ams-location') + '/get-paragraphs-editors.json',
									{},
									function(result) {
										for (var para_id in result) {
											if (!result.hasOwnProperty(para_id)) {
												continue;
											}
											var para = $('tr[data-ams-element-name="' + para_id + '"]', container);
											var editor = $('.editor', para);
											if (editor.is(':empty')) {
												editor.html(result[para_id]);
												MyAMS.initContent(editor);
											}
											$('.fa-plus-square-o', para).removeClass('fa-plus-square-o')
																		.addClass('fa-minus-square-o');
											para.data('ams-disabled-handlers', true);
										}
										if (!$('i.fa-plus-square-o', $('tbody', container)).exists()) {
											switcher.removeClass('fa-cog fa-spin')
													.addClass('fa-minus-square-o');
										}
									});
				} else {
					$('.editor', container).each(function() {
						MyAMS.skin.cleanContainer($(this));
						$(this).empty();
					});
					$('.fa-minus-square-o', container).removeClass('fa-minus-square-o')
													  .addClass('fa-plus-square-o');
					$('tr', container).removeData('ams-disabled-handlers');
				}
			},

			updateToolbar: function(settings) {
				var container = $('table[id="paragraphs_list"]');
				var para = $('tr[data-ams-element-name="' + settings.object_name + '"]', container);
				var toolbar = $('.title-toolbar', para);
				toolbar.replaceWith(settings.toolbar_tag);
				toolbar = $('.title-toolbar', para);
				MyAMS.initContent(toolbar);
			},

			updateMarkers: function(settings) {
				var container = $('table[id="paragraphs_list"]');
				var para = $('tr[data-ams-element-name="' + settings.object_name + '"]', container);
				var toolbar = $('.title-toolbar', para);
				var marker = $('DIV.action.' + settings.marker_type, toolbar);
				if (marker.exists()) {
					marker.replaceWith(settings.marker_tag);
				} else {
					$(settings.marker_tag).appendTo(toolbar);
				}
				if (settings.marker_tag) {
					marker = $('DIV.action.' + settings.marker_type, toolbar);
					MyAMS.initContent(marker);
				}
				MyAMS.helpers.sort(toolbar, 'weight');
			}
		},


		/**
		 * Illustrations management
		 */
		illustration: {

			addIllustration: function() {
				var source = $(this);
				var group = source.parents('.btn-group');
				var legend = group.siblings('legend.switcher');
				$('i.fa-plus', legend).click();
				source.hide();
			}
		},


		/**
		 * Form fields management
		 */
		fields: {

			refreshField: function(changes) {
				var container = $('table[id="form_fields_list"]');
				var para = $('tr[data-ams-element-name="' + changes.object_name + '"]', container);
				$('td:nth-child(4)', para).html(changes.title);
			},

			switchVisibility: function (element) {
				return function () {
					var source = $(this);
					var association = source.parents('tr');
					var container = association.parents('table');
					MyAMS.ajax.post(container.data('ams-location') + '/set-form-field-visibility.json',
						{object_name: association.data('ams-element-name')},
						function (result, status) {
							if (result.visible) {
								$('i', source).attr('class', 'fa fa-fw fa-eye');
							} else {
								$('i', source).attr('class', 'fa fa-fw fa-eye-slash text-danger');
							}
						});
				};
			}
		},


		/**
		 * Themes management
		 */
		themes: {

			initExtracts: function(element) {
				var thesaurus = $('select[name="form.widgets.thesaurus_name:list"]', element);
				var thesaurus_name = thesaurus.val();
				var extract = $('select[name="form.widgets.extract_name:list"]', element);
				var extract_name = extract.val();
				if (thesaurus_name) {
					MyAMS.jsonrpc.post('getExtracts',
									   {thesaurus_name: thesaurus_name},
									   {url: '/api/thesaurus/json'},
									   function(data) {
											extract.empty();
											$(data.result).each(function() {
												$('<option></option>').attr('value', this.id)
																	  .attr('selected', this.id === extract_name)
																	  .text(this.text)
																	  .appendTo(extract);
											});
									   });
				}
				extract.attr('data-ams-events-handlers', '{"select2-open": "PyAMS_content.themes.getExtracts"}');
			},

			getExtracts: function(event) {
				var select = $(event.currentTarget);
				var form = select.parents('form');
				var thesaurus_name = $('select[name="form.widgets.thesaurus_name:list"]', form).val();
				if (thesaurus_name) {
					MyAMS.jsonrpc.post('getExtracts',
									   {thesaurus_name: thesaurus_name},
									   {url: '/api/thesaurus/json'},
									   function(data) {
											var extract = $('select[name="form.widgets.extract_name:list"]', form);
											var plugin = extract.data('select2');
											plugin.results.empty();
											plugin.opts.populateResults.call(plugin, plugin.results, data.result,
																			 {term: ''});
									   });
				}
			}
		},


		/**
		 * Review comments management
		 */
		review: {

			timer: null,
			timer_duration: {
				general: 30000,
				chat: 5000
			},

			initComments: function(element) {
				var chat = $('.chat-body', element);
				chat.animate({scrollTop: chat[0].scrollHeight}, 1000);
				clearInterval(PyAMS_content.review.timer);
				PyAMS_content.review.timer = setInterval(PyAMS_content.review.updateComments,
														 PyAMS_content.review.timer_duration.chat);
				MyAMS.skin.registerCleanCallback(PyAMS_content.review.cleanCommentsCallback);
			},

			cleanCommentsCallback: function() {
				clearInterval(PyAMS_content.review.timer);
				PyAMS_content.review.timer = setInterval(PyAMS_content.review.updateComments,
														 PyAMS_content.review.timer_duration.general);
			},

			updateComments: function() {
				var badge = $('.badge', 'nav a[href="#review-comments.html"]'),
					count;
				var chat = $('.chat-body', '.widget-body');
				if (chat.exists()) {
					count = $('.message', chat).length;
				} else {
					count = parseInt(badge.text());
				}
				MyAMS.ajax.post('get-last-review-comments.json', {count: count}, function(result) {
					if (chat.exists()) {
						badge.removeClass('bg-color-danger')
							 .addClass('bg-color-info');
					}
					if (count !== result.count) {
						badge.text(result.count).removeClass('hidden');
						if (chat.exists()) {
							$('.messages', chat).append(result.content);
							chat.animate({scrollTop: chat[0].scrollHeight}, 1000);
						}
						if (!chat.exists()) {
							badge.removeClass('bg-color-info')
								 .addClass('bg-color-danger')
								 .animate({'padding': '3px 12px 2px',
										   'margin-right': '9px'}, 'slow', function() {
									$(this).animate({'padding': '3px 6px 2px',
													 'margin-right': '15px'}, 'slow');
								 });
						}
					}
				});
			},

			initCommentData: function(veto) {
				var chat = $('.chat-body', '.widget-body');
				return {count: $('.message', chat).length};
			},

			addCommentAction: function() {
				return function() {
					$('textarea[name="comment"]').focus();
				};
			},

			addCommentCallback: function(options) {
				var form = $(this);
				var widget = form.parents('.widget-body');
				$('.messages', widget).append(options.content);
				$('textarea[name="comment"]', form).val('');
				var chat = $('.chat-body', widget);
				chat.animate({scrollTop: chat[0].scrollHeight}, 1000);
				$('.badge', 'nav a[href="#review-comments.html"]').text(options.count).removeClass('hidden');
			}
		}
	};

	globals.PyAMS_content = PyAMS_content;

	$(globals.document).on('PyAMS_content.changed_item', function(event, settings) {
		MyAMS.executeFunctionByName(settings.handler, document, settings);
	});

	var badge = $('.badge', 'nav a[href="#review-comments.html"]');
	if (badge.exists()) {
		PyAMS_content.review.timer = setInterval(PyAMS_content.review.updateComments,
												 PyAMS_content.review.timer_duration.general);
	}

})(jQuery, this);
